/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "monitor.h"
#include "protocol.h"
#include "ssl.h"


/* Private prototypes */
static void _gcppl(Port_T *);
static void _gcpl(Service_T *);
static void _gcpql(Resource_T *);
static void _gcppil(ProcInfo_T *);
static void _gcptl(Timestamp_T *);
static void _gccmd(Command_T *);
static void _gcpdil(DeviceInfo_T *);
static void _gcpdl(Dependant_T *);
static void _gcso(Size_T *);
static void _gcchecksum(Checksum_T *);
static void _gcperm(Perm_T *);
static void _gcuid(Uid_T *);
static void _gcgid(Gid_T *);


/**
 *  Release allocated memory.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Martin Pala, <martin.pala@iol.cz>
 *
 *  @version \$Id: gc.c,v 1.30 2003/08/13 22:18:42 chopp Exp $
 *
 *  @file
 */


/* ------------------------------------------------------------------ Public */


void gc() {

  gc_protocols();
  if(servicelist) _gcpl(&servicelist);
  if(Run.mygroup) free(Run.mygroup);
  
}


void gc_service(Service_T *s) {

  ASSERT(s&&*s);
  
  if((*s)->portlist) {
    _gcppl(&(*s)->portlist);
  }

  if((*s)->maillist) {
    gc_mail_list(&(*s)->maillist);
  }

  if((*s)->resourcelist) {
    _gcpql(&(*s)->resourcelist);
  }

  if((*s)->procinfo) {
    _gcppil(&(*s)->procinfo);
  }
  
  if((*s)->devinfo) {
    _gcpdil(&(*s)->devinfo);
  }
  
  if((*s)->timestamplist) {
    _gcptl(&(*s)->timestamplist);
  }

  if((*s)->sizelist) {
    _gcso(&(*s)->sizelist);
  }

  if((*s)->checksum) {
    _gcchecksum(&(*s)->checksum);
  }

  if((*s)->perm) {
    _gcperm(&(*s)->perm);
  }

  if((*s)->uid) {
    _gcuid(&(*s)->uid);
  }

  if((*s)->gid) {
    _gcgid(&(*s)->gid);
  }

  if((*s)->dependantlist) {
    _gcpdl(&(*s)->dependantlist);
  }

  if((*s)->start) {
    _gccmd(&(*s)->start);
  }
  
  if((*s)->stop) {
    _gccmd(&(*s)->stop);
  }
  
  free((*s)->name);
  free((*s)->path);
  free((*s)->group);
  
  (*s)->next= NULL;

  pthread_mutex_destroy(&(*s)->mutex);

  free(*s);

  *s= NULL;

}
  

void gc_mail_list(Mail_T *m) {

  ASSERT(m);
  
  if((*m)->next)
    gc_mail_list(&(*m)->next);
  
  free((*m)->to);
  free((*m)->from);
  free((*m)->subject);
  free((*m)->message);
  free((*m)->opt_message);
  free(*m);
  *m= NULL;
  
}

    
/* ----------------------------------------------------------------- Private */


static void _gcpl(Service_T *s) {
  
  ASSERT(s&&*s);

  if((*s)->next) {
    _gcpl(&(*s)->next);
  }
  
  gc_service(&(*s));
  *s= NULL;
    
}


static void _gccmd(Command_T *c) {

  int i;
  
  ASSERT(c&&*c);
  
  for(i= 0; (*c)->arg[i]; i++)
    free((*c)->arg[i]);
  free(*c);
  *c= NULL;

}



static void _gcppl(Port_T *p) {
  
  ASSERT(p&&*p);

  if((*p)->next) {
    _gcppl(&(*p)->next);
  }

  if((*p)->exec) {
    _gccmd(&(*p)->exec);
  }

  free((*p)->hostname);
  free((*p)->request);
  free((*p)->pathname);
  free((*p)->address);
  free((*p)->SSL.certmd5);
  free(*p);
  *p= NULL;

}


static void _gcpql(Resource_T *q) {

  ASSERT(q);

  if((*q)->next) {
    _gcpql(&(*q)->next);
  }
  
  if((*q)->exec) {
    _gccmd(&(*q)->exec);
  }

  free(*q);
  *q= NULL;
  
}


static void _gcppil(ProcInfo_T *pi) {

  ASSERT(pi);

  free(*pi);
  *pi= NULL;

}


static void _gcpdil(DeviceInfo_T *di) {

  ASSERT(di);

  free(*di);
  *di= NULL;

}


static void _gcptl(Timestamp_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    _gcptl(&(*p)->next);
  }

  if((*p)->exec) {
    _gccmd(&(*p)->exec);
  }
  free(*p);
  *p= NULL;

}


static void _gcso(Size_T *s) {
  
  ASSERT(s);

  if((*s)->next)
    _gcso(&(*s)->next);

  if((*s)->exec)
    _gccmd(&(*s)->exec);

  free(*s);
  *s= NULL;

}


static void _gcchecksum(Checksum_T *s) {
  
  ASSERT(s);

  if((*s)->exec)
    _gccmd(&(*s)->exec);

  free((*s)->md5);
  free(*s);
  *s= NULL;

}


static void _gcperm(Perm_T *s) {
  
  ASSERT(s);

  if((*s)->exec)
    _gccmd(&(*s)->exec);

  free(*s);
  *s= NULL;

}


static void _gcuid(Uid_T *s) {
  
  ASSERT(s);

  if((*s)->exec)
    _gccmd(&(*s)->exec);

  free(*s);
  *s= NULL;

}


static void _gcgid(Gid_T *s) {
  
  ASSERT(s);

  if((*s)->exec)
    _gccmd(&(*s)->exec);

  free(*s);
  *s= NULL;

}


static void _gcpdl(Dependant_T *d) {

  ASSERT(d);

  if((*d)->next) {
    _gcpdl(&(*d)->next);
  }

  free((*d)->dependant);
  free(*d);
  *d= NULL;

}
