/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif 

#ifdef HAVE_SETJMP_H
#include <setjmp.h>
#endif 

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "monitor.h"
#include "net.h"
#include "socket.h"


/**
 *  Connect to a SMTP server and send mail.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *
 *  @version \$Id: sendmail.c,v 1.20 2003/08/13 22:18:43 chopp Exp $
 *
 *  @file
 */


/* ------------------------------------------------------- Private variables */


static Socket_T S;
static char *server;
static sigjmp_buf error;


/* -------------------------------------------------------------- Prototypes */


static void do_status();
static void finalize_server();
static int  initialize_server();
static void do_send(const char *, ...);


/* ------------------------------------------------------------------ Public */


/**
 * Send SMTP mail messages
 * @param mail A Mail object
 */
void sendmail(Mail_T mail) {

  Mail_T m;
  char localhost[STRLEN];
  char *now = get_RFC822date(NULL);
  
  ASSERT(mail);

  if(gethostname(localhost, sizeof(localhost)) < 0) {
    snprintf(localhost, STRLEN, LOCALHOST);
  }

  
  if(!initialize_server()) {
    goto exit;
  }
  
  if(sigsetjmp(error, TRUE)) {
    goto exit;
  }

  do_status();
  do_send("HELO %s\r\n", localhost); 
  do_status();

  for(m= mail; m; m= m->next) { 
    
    do_send("MAIL FROM: <%s>\r\n", m->from);
    do_status();
    do_send("RCPT TO: <%s>\r\n", m->to);
    do_status();
    do_send("DATA\r\n");
    do_status();
    do_send("From: %s\r\n", m->from);
    do_send("To: %s\r\n", m->to);
    do_send("Subject: %s\r\n", m->subject);
    do_send("Date: %s\r\n", now);
    do_send("X-Mailer: %s %s\r\n", prog, VERSION);
    do_send("Mime-Version: 1.0\r\n");
    do_send("Content-Type: text/plain; charset=\"iso-8859-1\"\r\n");
    do_send("Content-Transfer-Encoding: quoted-printable\r\n");
    do_send("\r\n");
    do_send("%s\r\n", m->message);
    if(m->opt_message) {
      do_send("%s\r\n", m->opt_message);
    }
    do_send(".\r\n");
    do_status();
    
  }

  do_send("QUIT\r\n");
  
  exit:
  free(now);
  finalize_server();
  
}


/* ----------------------------------------------------------------- Private */


static int initialize_server() {
  
  S= NULL;
  server= Run.mailserver?Run.mailserver:LOCALHOST;
  
  S= socket_new(server, SMTP_PORT, SOCKET_TCP, FALSE);
  if(!S) {
    log("Cannot open a connection to the mailserver '%s' -- %s\n",
	server, STRERROR);
    return FALSE;
  }
  
  return TRUE;
  
}


static void finalize_server() {
  
  if(S) {
    socket_free(&S);
  }
  
}


void do_send(const char *format, ...) {
  
  va_list ap;
  char msg[2048];
  
  va_start(ap,format);
  vsnprintf(msg, 2048, format, ap);
  va_end(ap);
  
  if(socket_write(S, msg, strlen(msg)) <= 0) {
    
    log("Sendmail: error sending data to the server '%s' -- %s\n",
	  server, STRERROR);
    siglongjmp(error, TRUE);
    
  }
  
}


static void do_status() {
  
  int  status;
  char buf[STRLEN];
  
  if(socket_read(S, buf, sizeof(buf)) <= 0) {
    
    log("Sendmail: error receiving data from the mailserver '%s' -- %s\n",
	server, STRERROR);
    siglongjmp(error, TRUE);
    
  }
  
  chomp(buf);
  
  sscanf(buf, "%d", &status);
  if(status >= 400) {
    
    log("Sendmail error: %s\n", buf);
    siglongjmp(error, TRUE);
    
  }
  
}


